package de.hsel.spm.baudas.web;

import com.google.gson.Gson;
import de.hsel.spm.baudas.analysis.TopFlopArticle;
import org.jetbrains.annotations.NotNull;

import javax.servlet.annotation.WebServlet;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.io.IOException;
import java.io.PrintWriter;
import java.nio.charset.StandardCharsets;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.UUID;
import java.util.*;
import java.util.regex.Pattern;

/**
 * top/flop articles list diagram.
 *
 * @author Johannes Theiner
 * @version 0.1
 * @since 0.1
 **/

@WebServlet("/top_flop")
public class TopFlopArticleDiagram extends HttpServlet {

    private static Map<UUID, Map> cache = new HashMap<>();

    private static final long serialVersionUID = 6567531464214L;

    @Override
    protected void doGet(@NotNull HttpServletRequest req, @NotNull HttpServletResponse resp) throws IOException {
        req.setCharacterEncoding(StandardCharsets.UTF_8.name());
        resp.setCharacterEncoding(StandardCharsets.UTF_8.name());
        resp.setContentType("application/json");
        PrintWriter out = resp.getWriter();

        Gson gson = new Gson();

        String id = req.getParameter("id");
        boolean match = Pattern.matches("[a-fA-F0-9]{8}-[a-fA-F0-9]{4}-[a-fA-F0-9]{4}-[a-fA-F0-9]{4}-[a-fA-F0-9]{12}", id);
        if(id == null || !match) {
            return;
        }

        UUID uuid = UUID.fromString(id);

        if(cache.containsKey(uuid)) {
            out.println(gson.toJson(cache.get(uuid)));
            return;
        }

        File file = Data.get(uuid);

        TopFlopArticle articles = new TopFlopArticle(file);
        Map<String, Integer> map = articles.getResult();

        Map<String, List<String>> result = new HashMap<>();

        List<String> labels = new ArrayList<>();
        List<String> data = new ArrayList<>();

        for (Map.Entry<String, Integer> entry : map.entrySet()) {
            labels.add(entry.getKey());
            data.add(entry.getValue().toString());
        }

        result.put("labels", labels);
        result.put("data", data);


        cache.put(uuid, result);
        out.print(gson.toJson(result));
    }
}