package de.hsel.spm.baudas.web;

import com.google.gson.Gson;
import de.hsel.spm.baudas.analysis.WeekOverview;
import org.jetbrains.annotations.NotNull;

import javax.servlet.annotation.WebServlet;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.io.IOException;
import java.io.PrintWriter;
import java.nio.charset.StandardCharsets;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Comparator;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.UUID;
import java.util.regex.Pattern;

/**
 * changes data from week overview into readable format for chart.js
 *
 * @author Johannes Theiner
 * @version 0.1
 * @since 0.1
 **/
@WebServlet("/week_overview")
public class WeekOverviewDiagram extends HttpServlet {

    private static Map<UUID, Map> cache = new HashMap<>();

    private static final long serialVersionUID = 8484151844118L;


    @Override
    protected void doGet(@NotNull HttpServletRequest req, @NotNull HttpServletResponse resp) throws IOException {
        req.setCharacterEncoding(StandardCharsets.UTF_8.name());
        resp.setCharacterEncoding(StandardCharsets.UTF_8.name());
        resp.setContentType("application/json");
        PrintWriter out = resp.getWriter();

        List<String> definedOrder = Arrays.asList("Montag", "Dienstag", "Mittwoch", "Donnerstag", "Freitag", "Samstag");
        Map<String, List<String>> result = new HashMap<>();

        Gson gson = new Gson();

        String id = req.getParameter("id");
        boolean match = Pattern.matches("[a-fA-F0-9]{8}-[a-fA-F0-9]{4}-[a-fA-F0-9]{4}-[a-fA-F0-9]{4}-[a-fA-F0-9]{12}", id);


        if (id == null || !match) {
            return;
        }

        UUID uuid = UUID.fromString(id);
        if(cache.containsKey(uuid)) {
            out.println(gson.toJson(cache.get(uuid)));
            return;
        }

        File file = Data.get(uuid);

        WeekOverview overview = new WeekOverview(file);

        List<String> count = new ArrayList<>();
        List<String> revenue = new ArrayList<>();

        List<String> labels = new ArrayList<>(overview.getResult().keySet());


        Comparator<String> comparator = Comparator.comparingInt(definedOrder::indexOf);
        labels.sort(comparator);

        for (String label : labels) {
            Map.Entry<Double, Integer> entry = overview.getResult().get(label);
            revenue.add(entry.getKey().toString());
            count.add(entry.getValue().toString());
        }


        result.put("labels", labels);
        result.put("count", count);
        result.put("revenue", revenue);

        cache.put(uuid, result);
        out.print(gson.toJson(result));
    }
}